//-----------------------------------------------------------------------------
// SimpleD3D
//
// This is a simple application that creates a window and handle its messages
// until the user decides to close it. It also initializes Direct3D with the
// help of Direct3DX and then renders a rotating triangle in the window. This
// is intended as an introduction to Direct3D, and nothing else.
//
// The following libraries must be linked with the project:
// user32.lib gdi32.lib d3dx.lib ddraw.lib winmm.lib
//
// Copyright (c) 2000, Andreas Jnsson 
//-----------------------------------------------------------------------------

// Don't include rarely used code to speed up compilation
#define WIN32_LEAN_AND_MEAN 

// We want to use the overloaded operators for common D3D types
#define D3D_OVERLOADS


#include <windows.h>
#include <mmsystem.h>
#include <d3dx.h>


//-----------------------------------------------------------------------------
// Vertex
// 
// This is the attributes we need for our vertices.
//-----------------------------------------------------------------------------
#define FVF_VERTEX (D3DFVF_XYZ|D3DFVF_DIFFUSE)
struct Vertex
{
	D3DVECTOR v;
	D3DCOLOR  c;
};

//-----------------------------------------------------------------------------
// Global variables
//-----------------------------------------------------------------------------
HWND              g_hWnd     = 0;
ID3DXContext     *g_pD3DX    = 0;
IDirect3DDevice7 *g_pD3DDev  = 0;
bool              g_bDXReady = false;



//-----------------------------------------------------------------------------
// Prototypes
//-----------------------------------------------------------------------------
HRESULT Create(HINSTANCE hInst);
int     Run();
LRESULT CALLBACK WndProc(HWND hWnd,UINT uMsg,WPARAM wParam,LPARAM lParam);
HRESULT InitializeD3D();
HRESULT IdleAction();
void    TerminateD3D();
void    ReportError(HRESULT hr);


//-----------------------------------------------------------------------------
// WinMain()
//
// This is the entry point for any Win32 application.
//-----------------------------------------------------------------------------
int WINAPI WinMain(HINSTANCE hInst, HINSTANCE hPrev, LPSTR lpCmdLine, int nShowCmd)
{
	// Create the window
	HRESULT hr;
	if( FAILED(hr = Create(hInst)) )
	{
		ReportError(hr);
		return 0;
	}

	// Initialize Direct3D
	if( FAILED(hr = InitializeD3D()) )
	{
		ReportError(hr);
		return 0;
	}

	// Run the application
	return Run();
}


//-----------------------------------------------------------------------------
// ReportError()
//-----------------------------------------------------------------------------
void ReportError(HRESULT hr)
{
	// Get a readable translation of the error
	char szError[256];
	D3DXGetErrorString(hr, 256, szError);

	// Show the error message to the user
	MessageBox(NULL, szError, "Fatal Error", MB_OK|MB_ICONERROR|MB_SYSTEMMODAL);
}



//-----------------------------------------------------------------------------
// Create()
//
// In this function we create the window that will be shown to the user
//-----------------------------------------------------------------------------
HRESULT Create(HINSTANCE hInst)
{
    // Register a window class
    WNDCLASS WndClass;
	WndClass.style         = CS_HREDRAW |      // Redraw if moved or resized
		                     CS_VREDRAW | 
							 CS_OWNDC;         // Unique device context 
	WndClass.lpfnWndProc   = (WNDPROC)WndProc; // Our message handler
	WndClass.cbClsExtra    = 0;                // No extra memory needed
	WndClass.cbWndExtra    = 0;                // No extra memory needed
	WndClass.hInstance     = hInst;            // App instance
	WndClass.hIcon         = LoadIcon(NULL, IDI_WINLOGO); // Standard icon
	WndClass.hCursor       = LoadCursor(NULL, IDC_ARROW); // Standard cursor
	WndClass.hbrBackground = (HBRUSH)GetStockObject(BLACK_BRUSH);  
	WndClass.lpszMenuName  = NULL;             // No menu
	WndClass.lpszClassName = "Application Window"; 

    if( !RegisterClass( &WndClass ) )
		return E_FAIL;

    // Create the render window
    g_hWnd = CreateWindow( "Application Window", // The window class 
	                       "Simple Direct3D",    // The window title
	  				       WS_OVERLAPPEDWINDOW,  // Normal window style
						   CW_USEDEFAULT,        // Default position
						   CW_USEDEFAULT,
			  		   	   640,                  // The size of the window
						   480,
						   NULL,                 // No parent window
						   NULL,                 // No menu
						   hInst,                // The app instance
						   NULL );               // No extra info
	if( g_hWnd == NULL )
		return E_FAIL;

	// Show the window
	ShowWindow( g_hWnd, SW_SHOW );
    UpdateWindow( g_hWnd );

	return S_OK;
}




//-----------------------------------------------------------------------------
// Run()
// 
// Here we receive the messages that are sent to us from the OS
//-----------------------------------------------------------------------------
int Run()
{
	// Check messages until we receive a WM_QUIT message
    MSG Msg;
	Msg.message = ~WM_QUIT; // Make sure it is not WM_QUIT
	while( Msg.message != WM_QUIT )
    {
		// We will use PeekMessage() so we can  
		// render the triangle during idle times
		if( PeekMessage( &Msg, NULL, 0, 0, PM_REMOVE ) )
		{
			// Dispatch the message to the window procedure
			DispatchMessage( &Msg );
		}
		else
		{
			// There were no messages in the queue  
			// so we can proceed with our rendering
			if( FAILED(IdleAction()) )
				PostQuitMessage(-1);
		}
    }

    return (int)Msg.wParam;
}




//-----------------------------------------------------------------------------
// WndProc()
//
// In the window procedure the messages are processed
//-----------------------------------------------------------------------------
LRESULT CALLBACK WndProc(HWND hWnd,UINT uMsg,WPARAM wParam,LPARAM lParam)
{
	HRESULT hr;

    switch( uMsg )
	{
	case WM_SIZE:
		// If the window is resized we need to resize the renderbuffers as well
		// otherwise Direct3D will stretch them to fit the window giving heavy
		// pixelating effects
		if( g_bDXReady )
		{
			if( FAILED(hr = g_pD3DX->Resize(LOWORD(lParam), HIWORD(lParam))) )
			{
				ReportError(hr);
				g_bDXReady = false;
				PostQuitMessage(-1);
			}
		}
		return 0;

	case WM_DESTROY:
		// Terminate the Direct3D objects before the window is deleted
		TerminateD3D();

		// Tell the message loop to quit
		PostQuitMessage(0);
		return 0;
	}

	// Let the default window procedure handle 
	// any message that we don't care about
    return DefWindowProc( hWnd, uMsg, wParam, lParam );
}


//-----------------------------------------------------------------------------
// InitializeD3D()
//-----------------------------------------------------------------------------
HRESULT InitializeD3D()
{
	// Begin by initializing D3DX
	HRESULT hr;
	if( FAILED(hr = D3DXInitialize()) )
		return hr;

	// Create the D3DX Context 
	// D3DX creates everything we need here
	hr = D3DXCreateContext(D3DX_DEFAULT, // Best acceleration available
		                   0,            // Windowed mode
                           g_hWnd,       // The render window
						   D3DX_DEFAULT, // Same width as window
						   D3DX_DEFAULT, // Same height as window
						   &g_pD3DX);    // The context
	if( FAILED(hr) )
		return hr;

	// In order to render with Direct3D we'll need the Direct3DDevice
	g_pD3DDev = g_pD3DX->GetD3DDevice();
	if( g_pD3DDev == 0 )
		return E_FAIL;

	// We will not use any lighting in this tutorial so we turn it off
	g_pD3DDev->SetRenderState(D3DRENDERSTATE_LIGHTING, FALSE);

	// Let's use another background color
	g_pD3DX->SetClearColor(D3DRGB(0.5f,0.5f,0.5f));

	// Success
	g_bDXReady = true;
	return S_OK;
}


//-----------------------------------------------------------------------------
// TerminateD3D()
//-----------------------------------------------------------------------------
void TerminateD3D()
{
	// Release our DirectX objects
	if( g_pD3DDev )
	{
		g_pD3DDev->Release();
		g_pD3DDev = 0;
	}

	if( g_pD3DX )
	{
		g_pD3DX->Release();
		g_pD3DX = 0;
	}

	// Let D3DX do some final uninitializations
	D3DXUninitialize();
}


//-----------------------------------------------------------------------------
// IdleAction()
//-----------------------------------------------------------------------------
HRESULT IdleAction()
{
	if( !g_bDXReady )
		return E_FAIL;

	// Clear the framebuffer and zbuffer
	HRESULT hr;
	hr = g_pD3DX->Clear(D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER);
	if( FAILED( hr ) )
		return hr;

	// Begin a new scene
    hr = g_pD3DDev->BeginScene();
	if( SUCCEEDED( hr ) )
	{
		// Let's rotate the triangle at a steady rate
		D3DXMATRIX Rot;
		D3DXMatrixRotationZ(&Rot, timeGetTime()/1000.0f);
		g_pD3DDev->SetTransform(D3DTRANSFORMSTATE_WORLD, (D3DMATRIX*)&Rot);

		// Render the like sided triangle
		Vertex V[3];
		V[0].v = D3DVECTOR(-1,-0.577f,1); V[0].c = D3DRGB(1,0,0); // Red
		V[1].v = D3DVECTOR( 0, 1.155f,1); V[1].c = D3DRGB(0,1,0); // Green
		V[2].v = D3DVECTOR( 1,-0.577f,1); V[2].c = D3DRGB(0,0,1); // Blue

		g_pD3DDev->DrawPrimitive(D3DPT_TRIANGLELIST,FVF_VERTEX,V,3,0);

		// The scene is finished
        hr = g_pD3DDev->EndScene();
	}
	if( FAILED(hr) )
		return hr;

	// Show the scene
    if( FAILED(hr = g_pD3DX->UpdateFrame(0)) )
		return hr;


	return S_OK;
}
